==> kmuddy-0.7.1.tar.gz: kmuddy-0.7.1/kmuddy/ctimerlist.cpp <==
/***************************************************************************
                          ctimerlist.cpp  -  description
                             -------------------
    begin                : St mj 8 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@lkmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctimerlist.h"
#include "ctimer.h"
#include "dialogs/dlgtimers.h"

#include <qtimer.h>
#include <kconfig.h>

cTimerList::cTimerList (QString filename, cSession *_sess) :
    cSaveableList (_sess)
{
  //sess is assigned in cSaveableList's constructor
  prototype = new cTimer (sess);
  timerTick = new cTimerTick (sess);
  objName = "Timer";
  config = new KConfig (filename);
  load ();
  timerTick->load( config, "" );
  tickCount = 0;
  timer = new QTimer;
  connect (timer, SIGNAL (timeout ()), this, SLOT (timeout ()));
}

cTimerList::~cTimerList ()
{
  save ();
  config->sync ();
  timerTick->save( config, "" );

  disableTimers ();
  delete timer;
  delete timerTick;
}

void cTimerList::enableTimers ()
{
  //reset tick counts first
  tickCount = 0;
  for (reset(); *this; (*this)++)
    ((cTimer *) cur)->resetTickCount();
  timerTick->resetTickCount();
  timer->start (1000);
  _enabled = true;
}

void cTimerList::disableTimers ()
{
  timer->stop ();
  _enabled = false;
}

void cTimerList::timeout ()
{
  //update tick count
  tickCount++;
  //!!! this does not check for overflow, therefore could stop working after
  //about 49 days;

  timerTick->tick (tickCount);
  //advance through the list of timers, asking each of them to do its task :)
  for (reset(); *this; (*this)++)
  {
    cTimer *t = (cTimer *) cur;
    t->tick (tickCount);
  }

  //remove single-shot timers when they got executed
  reset ();
  while (*this)
  {
    cTimer *t = (cTimer *) cur;
    if (t->shouldDelete ())
    {
      //we cannot use removeCurrent, because it resets current item. Nothing
      //cricital, just a bit innefficient...
      setMarker ();
      (*this)++;
      removeMarked ();
      if (dialog)
        dialog->updateMe ();
    }
    else
      (*this)++;
  }    
}

==> kmuddy-0.8.tar.gz: kmuddy-0.8/kmuddy/ctimerlist.cpp <==
/***************************************************************************
                          ctimerlist.cpp  -  list of timers
                             -------------------
    begin                : St maj 8 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctimerlist.h"
#include "ctimer.h"
#include "ctimertick.h"
#include "dialogs/dlgtimers.h"

#include <qtimer.h>
#include <kconfig.h>

cTimerList::cTimerList (QString filename, int sess) :
    cSaveableList ("timerlist", sess)
{
  //sess is assigned in cSaveableList's constructor
  prototype = new cTimer (sess);
  timerTick = new cTimerTick (sess);
  objName = "Timer";
  config = new KConfig (filename);
  load ();
  timerTick->load( config, "" );
  tickCount = 0;
  timer = new QTimer;
  connect (timer, SIGNAL (timeout ()), this, SLOT (timeout ()));
}

cTimerList::~cTimerList ()
{
  save ();
  config->sync ();
  timerTick->save( config, "" );

  disableTimers ();
  delete timer;
  delete timerTick;
}

void cTimerList::enableTimers ()
{
  //reset tick counts first
  tickCount = 0;
  for (reset(); *this; (*this)++)
    ((cTimer *) cur)->resetTickCount();
  timerTick->resetTickCount();
  timer->start (1000);
  _enabled = true;
}

void cTimerList::disableTimers ()
{
  timer->stop ();
  _enabled = false;
}

void cTimerList::timeout ()
{
  //update tick count
  tickCount++;
  //!!! this does not check for overflow, therefore could stop working after
  //about 49 days;

  timerTick->tick (tickCount);
  //advance through the list of timers, asking each of them to do its task :)
  for (reset(); *this; (*this)++)
  {
    cTimer *t = (cTimer *) cur;
    t->tick (tickCount);
  }

  //remove single-shot timers when they got executed
  reset ();
  while (*this)
  {
    cTimer *t = (cTimer *) cur;
    if (t->shouldDelete ())
    {
      //we cannot use removeCurrent, because it resets current item. Nothing
      //cricital, just a bit innefficient...
      setMarker ();
      (*this)++;
      removeMarked ();
      if (dialog)
        dialog->updateMe ();
    }
    else
      (*this)++;
  }    
}
